//
// simple-display-table.cxx -- Use Fl_Table to display simple array of floats
//
// erco 05/12/06
//

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Table_Row.H>

#include <stdio.h>
#include <stdlib.h>	// rand
#include <math.h>	// sin/cos

#define ROWS 40		// how many rows high
#define COLS 10		// how many columns across

class FloatTable : public Fl_Table_Row
{
    float data[ROWS][COLS];						// data array for the table

protected:
    void draw_cell(TableContext context, int R=0, int C=0, int X=0, int Y=0, int W=0, int H=0)
    {
	static char s[40];
	switch ( context )
	{
	    case CONTEXT_STARTPAGE:
		fl_font(FL_HELVETICA, 16);				// font to use for all cells
		return;

	    case CONTEXT_COL_HEADER:
		sprintf(s, "Column %d", C);				// text to show at top of each column
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, color());	// solid 'up box'
		fl_color(FL_BLACK);
		fl_draw(s, X, Y, W, H, FL_ALIGN_CENTER);		// draw text centered in box
		return;

	    case CONTEXT_ROW_HEADER:
		sprintf(s, "%d", R);					// text to show at left of each row
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, color());	// solid 'up box'
		fl_color(FL_BLACK);
		fl_draw(s, X, Y, W, H, FL_ALIGN_CENTER);		// draw text centered in box
		return;

	    case CONTEXT_CELL:
	    	sprintf(s, "%.2f", data[R][C]);				// text to show in each cell
		fl_push_clip(X, Y, W, H);				// make sure our text doesn't leak outside of XYWH
		{
		    fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, color());	// solid 'up box'
		    fl_color(FL_BLACK);
		    fl_draw(s, X, Y, W-4, H, FL_ALIGN_RIGHT);		// draw text right-aligned with 4 pixel margin
		}
		fl_pop_clip();
		return;

	    default:
		return;
	}
    }

public:
    FloatTable(int x, int y, int w, int h, const char *l=0) : Fl_Table_Row(x,y,w,h,l)
    {
        // Tell Fl_Table how many rows/cols of data
	rows(ROWS);
	cols(COLS);

	// Enable row/col headers
	row_header(1);
	col_header(1);

	// Enable row/col resizing
	col_resize(4);
	row_resize(4);

	end();
    }
    ~FloatTable() { }

    void SetData(int R, int C, float val)
        { data[R][C] = val; }

    float GetData(int R, int C)
        { return(data[R][C]); }
};

int main()
{
    Fl_Window win(700, 400);
    FloatTable table(30, 30, win.w()-60, win.h()-60, "Display-Only Table");

    // Initialize the data array to contain some random data
    for ( int row=0; row<ROWS; row++ )
	for ( int col=0; col<COLS; col++ )
	    table.SetData(row, col, (float)(rand() % 1000) / 10.0);

    win.resizable(table);
    win.show();
    return(Fl::run());
}

